sbe.Book = sbe.Class.extend(
{
	data : null,
	con : null, // book container

	isSwipping : true,
	isFlipping : false,
	isLoading : false,

	currentPageIndex : -1, // -1 is the startup/loader
	currentPageObj : null,
	pageObjs : [],

	init : function(data)
	{
		sbe.book = this;

		// keep hold of the data
		this.data = data;
		

		// add this container to the stage
		this.con = new cjs.Container();
		this.con.on('click', this.interactionHandler, this);
		this.con.on('mousedown', this.interactionHandler, this);
		this.con.on('mouseover', this.interactionHandler, this);
		this.con.on('mouseout', this.interactionHandler, this);
		this.con.on('pressup', this.interactionHandler, this);
		this.con.mouseEnabled = true;
		sbe.system.stage.addChild( this.con );
        
        this.frameRateText = new createjs.Text("0", "20px Arial", "#39b4ed");
        if(data.fps) {
            sbe.system.stage.addChild(this.frameRateText);
        }

		// create the flip pages
		this.pageflipLeft = $('<div class="pageflip left"><div class="gradient"></div></div>');
		this.pageflipRight = $('<div class="pageflip right"><div class="gradient"></div></div>');
		this.pageflipLeft.css('display', 'none');
		this.pageflipRight.css('display', 'none');
		$('#' + data.wrapperId).append(this.pageflipLeft, this.pageflipRight);

		// add the buttons on the side
		this.pageturnLeft = $('<img class="pageturn left" src="' + data.pageTurnBtns[0] + '" />').css('display', 'none');
		this.pageturnRight = $('<img class="pageturn right" src="' + data.pageTurnBtns[1] + '" />').css('display', 'none');
		this.pageturnReplay = $('<img class="pageturn right" src="' + data.pageReplayBtn + '" />').css('display', 'none');
		$('#' + data.containerId).append(this.pageturnLeft, this.pageturnRight, this.pageturnReplay);

		// clicks for the page turns
		this.pageturnLeft.on('click', function()
		{
			if ( !sbe.book.isFlipping )
				if ( $(this).css('opacity') === '1' )
					sbe.book.gotoPage(sbe.book.currentPageIndex - 1);
		});
		this.pageturnRight.on('click', function()
		{	
			if ( !sbe.book.isFlipping )
				if ( $(this).css('opacity') === '1' )
					sbe.book.gotoPage(sbe.book.currentPageIndex + 1);
		});
		// and the replay button
		this.pageturnReplay.on('click', function()
		{	
			sbe.book.pageturnReplay.hide();
			if ( !sbe.book.isFlipping )
				if ( $(this).css('opacity') === '1' )
					sbe.book.gotoPage(0);
				// sbe.book.currentPageIndex = 0;
		});

		// for the text replays
		$('#' + data.domId).on('click', 'img.replay', function(e)
		{
			var name = $(this).parent('div.text').selector[0].id;
			if ( sbe.book.currentPageObj )
				sbe.book.currentPageObj.voText( name.split('vo-').pop() );
		});

		// create the pages
		for ( var i = 0; i < data.pages.length; i++ )
		{
			var pageObj = new sbe.Page(data.pages[i]);
			pageObj.n = i;
			this.pageObjs.push( pageObj );
		}

		// show the start up!
		// cjs.Tween.get(this).wait(200).call(this.showStartup, null, this);

		
	},

	runLoader: function () {
		this.updateLoader = function() {};
		sbe.loader.load(this.getAssetsToLoad(), function()
		{
			this.gotoPage( this.data.startOnPage );
			
		}.bind(this));
	},

	run : function()
	{
        this.frameRateText.text = ~~createjs.Ticker.getMeasuredFPS();
		this.con.x = sbe.system.width * 0.5;
		this.con.y = sbe.system.height * 0.5;

		if ( this.currentPageIndex < 0 )
			this.updateLoader();

		if ( this.currentPageObj )
			this.currentPageObj.update();

		if ( this.isSwipping )
			this.detectSwipping();
	},

	interactionHandler : function(e)
	{
		if ( !this.currentPageObj )
			return;

		this.currentPageObj[e.type](e);
	},

	canSwipe : function(flag)
	{
		this.isSwipping = flag;
	},

	detectSwipping : function()
	{
		if ( sbe.input.pressed(sbe.KEY.MOUSE1) )
		{
			this.touchStartX = sbe.input.mouse.x;
			this.touchStartY = sbe.input.mouse.y;
			this.touchTime = Date.now();
		}

		if ( sbe.input.released(sbe.KEY.MOUSE1) )
		{
			if ( Date.now() - this.touchTime < 600 )
			{
				var dist = Math.distance(sbe.input.mouse.x, sbe.input.mouse.y, this.touchStartX, this.touchStartY, true);
				var angle = Math.angle(sbe.input.mouse.x, sbe.input.mouse.y, this.touchStartX, this.touchStartY, true);

				if ( dist > 300 )
				{
					if ( angle < 0 )
						angle = 360 + angle;

					if ( angle < 45 || angle > 315 )
						this.gotoPage(this.currentPageIndex - 1);
					else if ( angle > 135 && angle < 225 )
						this.gotoPage(this.currentPageIndex + 1);
				} 
			}
		}
	},

	gotoPage : function(index)
	{
		if ( index < 0 || index > this.pageObjs.length - 1 )
			return;

		if ( this.isFlipping || this.isLoading )
			return;

		if ( this.data.gotoPage )
			this.data.gotoPage.call(this, index);

		this.isFlipping = true;

		// toggles the buttons on the side
		this.pageTurn(index);

		// end the current page
		if ( this.currentPageObj )
			this.currentPageObj.end();
		this.currentPageObj = null; // set to null so the update doesn't get called

		var oldPageObj = this.pageObjs[this.currentPageIndex],
			newPageObj = this.pageObjs[index],
			speed = this.data.pageFlipDuration,
			delay = this.data.pageFlipDelay;

		newPageObj.setup();
		newPageObj.con.visible = true;

		// turn page effect
		if ( this.currentPageIndex > -1 )
		{
			var o;
			var props;

			newPageObj.gradient.alpha = oldPageObj.gradient.alpha = 0;

			// we are on the cover, openning the book
			if ( this.currentPageIndex === 0 )
			{
				oldPageObj.prepTurn();
				newPageObj.prepLeft();

				newPageObj.con.addChild(newPageObj.gradient.set({ scaleX : 1, alpha : 0.7 }));
				cjs.Tween.get(newPageObj.gradient).wait(delay + speed).to({ alpha : 0 }, speed);

				// move the dom
				o = { v : 25 };
				props = { onChange : function() { this.css('left', o.v + '%'); }.bind(this.pageflipRight) };
				cjs.Tween.get(o, props).wait(delay).to({ v : 50 }, speed);
				this.pageflipRight.css('left', '25%');
				this.pageflipLeft.css('right', '50%');

				// translate
				var o2 = { v : -50 };
				var props2 = { onChange : function() { this.css('transform', 'translate(' + o2.v + '%, 0)'); }.bind($('#book-cover-front')) };
				cjs.Tween.get(o2, props2).wait(delay).to({ v : -7 }, speed).wait(speed).call(function()
				{
					$('#book-cover-front, #book-cover-full, #book-cover-top').toggle();
				});

				// move the page container
				oldPageObj.con.visible = false;
				newPageObj.con.x = newPageObj.mask.x = -sbe.system.pageWidth * 0.5;
				cjs.Tween.get(newPageObj.con).wait(delay - 25).to({ x : 0 }, speed);
				cjs.Tween.get(newPageObj.mask).wait(delay - 25).to({ x : 0 }, speed);

				this.pageFlip(1, oldPageObj.cacheCanvas, newPageObj.cacheCanvas);
			}
			// we are going to the cover
			else if ( index === 0 )
			{
				oldPageObj.prepLeft();
				newPageObj.prepTurn();

				oldPageObj.con.addChild(oldPageObj.gradient.set({ scaleX : 1, alpha : 0 }));
				cjs.Tween.get(oldPageObj.gradient).wait(delay).to({ alpha : 0.7 }, speed);

				// move the dom
				o = { v : 50 };
				props = { onChange : function() { this.css('left', o.v + '%'); }.bind(this.pageflipRight) };
				cjs.Tween.get(o, props).wait(speed + delay).to({ v : 25 }, speed);
				this.pageflipRight.css('left', '50%');
				this.pageflipLeft.css('right', '50%');

				// translate
				var o2 = { v : -7 };
				var props2 = { onChange : function() { this.css('transform', 'translate(' + o2.v + '%, 0)'); }.bind($('#book-cover-front')) };
				cjs.Tween.get(o2, props2).call(function()
				{
					$('#book-cover-front, #book-cover-full, #book-cover-top').toggle();
				}).wait(speed + delay).to({ v : -50 }, speed);

				// move the page container
				newPageObj.con.visible = false;
				cjs.Tween.get(oldPageObj.con).wait(speed + delay).to({ x : -sbe.system.pageWidth * 0.5 }, speed);
				cjs.Tween.get(oldPageObj.mask).wait(speed + delay).to({ x : -sbe.system.pageWidth * 0.5 }, speed);

				this.pageFlip(-1, newPageObj.cacheCanvas, oldPageObj.cacheCanvas);
			}
			// turn the page
			else
			{
				if ( index > this.currentPageIndex )
				{
					oldPageObj.prepRight();
					newPageObj.prepLeft();

					oldPageObj.con.addChild(oldPageObj.gradient.set({ scaleX : -1, alpha : 0 }));
					cjs.Tween.get(oldPageObj.gradient).wait(delay).to({ alpha : 0.7 }, speed);

					newPageObj.con.addChild(newPageObj.gradient.set({ scaleX : 1, alpha : 0.7 }));
					cjs.Tween.get(newPageObj.gradient).wait(delay + speed).to({ alpha : 0 }, speed);

					this.pageFlip(1, oldPageObj.cacheCanvas, newPageObj.cacheCanvas);
				}
				else if ( index < this.currentPageIndex )
				{
					oldPageObj.prepLeft();
					newPageObj.prepRight();

					oldPageObj.con.addChild(oldPageObj.gradient.set({ scaleX : 1, alpha : 0 }));
					cjs.Tween.get(oldPageObj.gradient).wait(delay).to({ alpha : 0.7 }, speed);

					newPageObj.con.addChild(newPageObj.gradient.set({ scaleX : -1, alpha : 0.7 }));
					cjs.Tween.get(newPageObj.gradient).wait(delay + speed).to({ alpha : 0 }, speed);

					this.pageFlip(-1, newPageObj.cacheCanvas, oldPageObj.cacheCanvas);
				}
			}

			cjs.Tween.get(this)
				.wait(delay)
				.call(cjs.Sound.play, ['sfx-page-turn'], cjs.Sound) // play turn sfx
				.wait((speed * 2) - (1000 / 60))
				.call(function(a, b)
				{ 
					a.con.mask = null; 
					a.con.visible = true;
					b.con.visible = false;

					// breakd down!
					b.breakdown();

					// start it
					// this.pageObjs[this.currentPageIndex].start();

				}, [newPageObj, oldPageObj], this)
				.wait(delay)
				.call(function(a)
				{
					// so it will update
					// this.currentPageObj = this.pageObjs[this.currentPageIndex];
					// this.currentPageObj.start();
					this.currentPageObj = a;
					a.start();

					// you can now flip the page!
					this.isFlipping = false;

				}, [newPageObj], this);
		}
		// on the cover
		else
		{
			this.currentPageObj = this.pageObjs[index];
			this.currentPageObj.start();
			this.isFlipping = false;
			$('#book-cover-front').css('display', 'block');
		}

		this.currentPageIndex = index;
		this.isLoading = true;

		// start loading next page assets
		sbe.loader.load(this.getAssetsToLoad(), this.assetsLoaded);
	},

	pageFlip : function(dir, rPage, lPage)
	{
		var ftn = function() { this.css('display', 'none'); },
			speed = this.data.pageFlipDuration,
			delay = this.data.pageFlipDelay,
			pfLeft = this.pageflipLeft,
			pfRight = this.pageflipRight,
			lGradient = pfLeft.find('.gradient'),
			rGradient = pfRight.find('.gradient');

		pfLeft.find('canvas').remove();
		pfRight.find('canvas').remove();

		lGradient.css('opacity', dir > 0 ? 0.7 : 0);
		rGradient.css('opacity', dir > 0 ? 0 : 0.7);

		pfLeft.prepend(lPage).css('display', 'block').animate({ rotateY : (dir > 0 ? 90 : 0) + 'deg' }, 0, null, function()
		{
			pfLeft.animate({ rotateY : (dir > 0 ? 0 : 90) + 'deg' }, speed, null, ftn.bind(pfLeft), dir > 0 ? speed + delay : delay);
			lGradient.animate({ opacity : dir > 0 ? 0 : 0.7 }, speed, null, null, dir > 0 ? speed + delay : delay);
		});

		pfRight.prepend(rPage).css('display', 'block').animate({ rotateY : '-' + (dir > 0 ? 0 : 90) + 'deg' }, 0, null, function()
		{
			pfRight.animate({ rotateY : '-' + (dir > 0 ? 90 : 0) + 'deg' }, speed, null, ftn.bind(pfRight), dir > 0 ? delay : speed + delay);
			rGradient.animate({ opacity : dir > 0 ? 0.7 : 0 }, speed, null, null, dir > 0 ? delay : speed + delay);
		});
	},

	pageTurn : function(index)
	{
		var speed = this.data.pageFlipDuration,
			delay = this.data.pageFlipDelay,
			ptLeft = this.pageturnLeft,
			ptRight = this.pageturnRight,
			ptReplay = this.pageturnReplay;
		// 	lastPage = index === sbe.book.pageObjs.length - 1;

		// if ( lastPage )
		// 	ptReplay.show().animate({ opacity : 1 }, speed);
		// else
		// 	ptReplay.animate({ opacity : 0 }, speed, ptReplay.hide.bind(ptReplay));

		// ptLeft.animate({ opacity : 0 }, delay, ptLeft.css.bind(ptLeft, 'display', 'none'));
		// ptRight.animate({ opacity : 0 }, delay, ptRight.css.bind(ptRight, 'display', 'none'));
		ptLeft.animate({ opacity : 0 }, delay);//, ptLeft.hide.bind(ptLeft));
		ptRight.animate({ opacity : 0 }, delay);//, ptRight.hide.bind(ptRight));
		ptReplay.animate({ opacity : 0 }, delay);//, ptReplay.hide.bind(ptReplay));
	},

	assetsLoaded : function()
	{
		if ( sbe.book.isFlipping )
		{
			setTimeout(sbe.book.assetsLoaded, 200);
			return;
		}

		sbe.book.isLoading = false;

		var index = sbe.book.currentPageIndex;
		var speed = sbe.book.data.pageFlipDuration;

		if ( index > 0 )
			sbe.book.pageturnLeft.show().animate({ opacity : 1 }, speed);
			// sbe.book.pageturnLeft.css('display', '').animate({ opacity : 1 }, speed);

		if ( index < sbe.book.pageObjs.length - 1 )
			sbe.book.pageturnRight.show().animate({ opacity : 1 }, speed);
		else
			sbe.book.pageturnReplay.show().animate({ opacity : 1 }, speed);
			// sbe.book.pageturnRight.css('display', '').animate({ opacity : 1 }, speed);
		
	},

	// loader stuff below

	showStartup : function()
	{
		var con = new cjs.Container().set({ name : 'startup', cursor : 'pointer' }),
			cSpr = new cjs.Shape( new cjs.Graphics().f('#fff').dc(0, 0, 128) ),
			aSpr = new cjs.Shape( new cjs.Graphics().f('#000').mt(-32, -64).lt(64, 0).lt(-32, 64) );

		con.addChild( cSpr, aSpr );
		this.con.addChild( con );

		con.on('click', this.hideStartup, this);
	},

	hideStartup : function()
	{
		this.con.removeChild( this.con.getChildByName('startup') );
		this.showLoader();
	},

	showLoader : function()
	{
		var con = new cjs.Container().set({ name : 'loader' }),
			bgSpr = new cjs.Shape( new cjs.Graphics().s('#fff').ss(4).f('#000').dr(0, 0, 200, 32) ).set({ x : -100, y : -16 }),
			bSpr = new cjs.Shape( new cjs.Graphics().f('#900').dr(0, 0, 196, 28) ).set({ x : -98, y : -14, scaleX : 0 });

		con.addChild( bgSpr, bSpr );
		this.con.addChild( con );

		this.status = 0;

		// start loading
		sbe.loader.load(this.getAssetsToLoad());
	},

	updateLoader : function()
	{
		if ( sbe.loader.status === 0 )
			return;

		this.status = this.status.lerp( sbe.loader.status, 0.08 );
		
		var con = this.con.getChildByName('loader');
		con.getChildAt(1).scaleX = this.status;

		// we've loaded enough
		if ( sbe.loader.status >= 1 && this.status >= 0.99 )
			this.hideLoader();
	},

	hideLoader : function()
	{
		this.con.removeChild( this.con.getChildByName('loader') );
		this.gotoPage( this.data.startOnPage );
	},

	getAssetsToLoad : function()
	{
		var assets = [];
		var pageN = this.currentPageIndex > -1 ? this.currentPageIndex : this.data.startOnPage;
		var asset;

		for ( var i = 0; i < this.data.assets.length; i++ )
		{
			asset = this.data.assets[i];

			// load the asset if we are one page away
			if ( typeof asset.page === 'number' )
			{
				if ( asset.page >= pageN - 1 && asset.page <= pageN + 1 )
					assets.push(asset);
			}
			// it belongs to multiple pages, so we need to check it
			else if ( asset.page instanceof Array  )
			{
				for ( var j = 0; j < asset.page.length; j++ )
				{
					if ( asset.page[j] >= pageN - 1 && asset.page[j] <= pageN + 1 )
						assets.push(asset);
				}
			}
			// no page indentifies? must be global, like vo
			else if ( typeof asset.page === 'undefined' )
				assets.push(asset);
		}

		// assets.push('')

		return assets;
	}
});