sbe.System = sbe.Class.extend(
{
	width : 0,
	height : 0,
	pageWidth : 0,
	pageHeight : 0,
	bookWidth : 0,
	bookHeight : 0,

	delta : 0,
	delegate : null, // book class
	stage : null, // the createjs stage

	hiddenEventType : null,
	// isWrongOrientation : false,

	staticInit : function()
	{
		return sbe.system || null;
	},

	init : function(bookClass, bookData)
	{
		sbe.system = this;

		// this.orientation = sbe.$('#' + bookData.orientationId);
		this.container = sbe.$('#' + bookData.containerId);
		this.wrapper = sbe.$('#' + bookData.wrapperId);
		this.canvas = sbe.$('#' + bookData.canvasId);
		this.dom = sbe.$('#' + bookData.domId);
		this.pageWidth = bookData.width;
		this.pageHeight = bookData.height;
		this.bookWidth = bookData.width * bookData.format;
		this.bookHeight = bookData.height;

		// setup the events
		this.setupEvents();

		// creates the createjs stage
		this.setupStage( this.canvas );

		// create the ticker
		createjs.Ticker.timingMode = createjs.Ticker.RAF;
		// createjs.Ticker.framerate = 12;
		createjs.Ticker.on('tick', this.run, this);

		// start it already
		this.delegate = new bookClass( bookData );
	},

	run : function(e)
	{
		if ( this.isFocused === -1 )
			return;

		if ( this.isFocused > 0 )
		{
			this.isFocused--;
			return;
		}

		this.delta = e.delta * 0.001;
		this.delegate.run();
		this.stage.update(e);

		sbe.input.clearPressed();
	},

	setupStage : function(canvas)
	{
		// create the stage
		var stage = new createjs.Stage( canvas );
		stage.preventSelection = false;
		stage.snapToPixelEnabled = true;

		// enable touch events
		if ( sbe.device.mobile && createjs.Touch.isSupported() )
		{
			createjs.Touch.enable(stage);
			stage.enableDOMEvents(false);
		}
		else
			stage.enableMouseOver(30);

		this.stage = stage;

		this.onWindowResize();
	},

	setupEvents : function()
	{
		var resizeWindowBound = this.onWindowResize.bind(this);
		var pageShowBound = this.onPageShow.bind(this);
		var pageHideBound = this.onPageHide.bind(this);
		var changeBound = this.onChange.bind(this);

		// some window events
		window.addEventListener('load', resizeWindowBound, false);
		window.addEventListener('resize', resizeWindowBound, false);
		window.addEventListener('orientationchange', this.onWindowResize.bind(this, true), false);
		window.addEventListener('pageshow', pageShowBound, false);
		window.addEventListener('pagehide', pageHideBound, false);
		window.addEventListener('focus', pageShowBound, false);
		window.addEventListener('blur', pageHideBound, false);

		var hidden = 'hidden';
		if ( hidden in document )
			document.addEventListener('visibilitychange', changeBound);
		else if ( (hidden = 'mozHidden') in document )
			document.addEventListener('mozvisibilitychange', changeBound);
		else if ( (hidden = 'webkitHidden') in document )
			document.addEventListener('webkitvisibilitychange', changeBound);
		else if ( (hidden = 'msHidden') in document )
			document.addEventListener('msvisibilitychange', changeBound);

		this.hiddenEventType = hidden;

		// prevent touch and dragging
		document.ontouchmove = function(e) { e.preventDefault(); };
	},

	_lastSize : 0,
	onWindowResize : function(force)
	{
		setTimeout(function() { if ( window.scrollTo ) window.scrollTo(0, 0); }, 0);

		if ( !force && this._lastSize === window.innerWidth + window.innerHeight )
			return;

		// var windowWidth = window.innerWidth,
		var windowWidth = (window.innerWidth - 185 > this.bookWidth ? this.bookWidth : window.innerWidth - 185).limit(144, Infinity),
			// windowHeight = window.innerHeight > this.bookHeight ? this.bookHeight : window.innerHeight,
			windowHeight = (window.innerHeight - 122 > this.bookHeight ? this.bookHeight : window.innerHeight - 122).limit(266, Infinity),

			widthScale = windowWidth / this.bookWidth,
			heightScale = windowHeight / this.bookHeight,

			canvasWidth = this.bookWidth,
			canvasHeight = this.bookHeight,

			wrapperStyle = this.wrapper.style,
			domStyle = this.dom.style,

			scale, w, h;

		if ( widthScale > heightScale ) 
		{
			if ( windowWidth > this.bookWidth * heightScale )
				canvasWidth = this.bookWidth;
			else if ( windowWidth < this.bookWidth * heightScale )
				canvasWidth = this.bookWidth;
			else
				canvasWidth = windowWidth / heightScale;
		}
		else
		{
			if ( windowHeight > this.bookHeight * widthScale )
				canvasHeight = this.bookHeight;
			else if ( windowHeight < this.bookHeight * widthScale )
				canvasHeight = this.bookHeight;
			else
				canvasHeight = windowHeight / widthScale;
		}

		// we want ints
		canvasWidth = ~~canvasWidth;
		canvasHeight = ~~canvasHeight;

		// this prevents flickering
		w = this.width;
		h = this.height;

		this.width = canvasWidth;
		this.height = canvasHeight;

		if ( w !== this.width || h !== this.height )
		{
			this.stage.canvas.width = this.width;
			this.stage.canvas.height = this.height;
		}

		// is the orientation wrong?
		// this.isWrongOrientation = windowWidth / windowHeight < canvasWidth / canvasHeight;

		// get aspect scale
		scale = widthScale > heightScale ? windowHeight / canvasHeight : windowWidth / canvasWidth;
		if(windowWidth <512 && windowWidth > 370){

			scale = scale +0.05;	
		} else if(windowWidth < 370){

			scale = scale +0.04;	
		} else if(windowWidth > 512){

			scale = scale +0.1;
		}
		



		// set some styles to the wrapper
		wrapperStyle.width = ~~(canvasWidth * scale) + 'px';
		wrapperStyle.height = ~~(canvasHeight * scale) + 'px';
		wrapperStyle.marginTop = ~~((window.innerHeight - (canvasHeight * scale)) * 0.5) + 'px';

		domStyle.transform = domStyle['-webkit-transform'] = domStyle['-ms-transform'] = 'scale(' + scale + ')';

		this.container.style.fontSize = (parseInt(wrapperStyle.height) / this.pageHeight) + 'px';
		// this.orientation.style.display = this.isWrongOrientation ? 'block' : 'none';

		// set the last size
		this._lastSize = window.innerWidth + window.innerHeight;
	},

	_wasMuted : false,
	onPageShow : function() 
	{
		if ( !this.isFocused )
			return;

		this.isFocused = 1;
		cjs.Sound.muted = this._wasMuted;
	},

	onPageHide : function() 
	{ 
		if ( this.isFocused )
			return;

		this.isFocused = -1;
		this._wasMuted = cjs.Sound.muted;
		cjs.Sound.muted = true;
	},

	onChange : function(e)
	{
		if ( document[this.hiddenEventType] )
			this.onPageHide();
		else
			this.onPageShow();
	}
});